@@ -1,3 +1,4 @@
+#    $Id: Nominet.pm,v 1.4 2013/11/06 10:23:29 pete Exp $
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -28,15 +29,18 @@ use constant EPP_XMLNS	=> 'urn:ietf:params:xml:ns:epp-1.0';
 use vars qw($Error $Code $Message);
 
 BEGIN {
-	our $VERSION = '0.11_01';
+	use Exporter ();
+	our ($VERSION, @ISA, @EXPORT, @EXPORT_OK);
+	$VERSION    = '0.01';
+	@ISA        = qw(Net::EPP::Simple Exporter);
+	@EXPORT     = qw();
+	@EXPORT_OK  = qw();
 }
+our @EXPORT_OK;
 
 # file-scoped lexicals
-my %Host      = (
-	prod => 'epp.nominet.org.uk',
-	test => 'testbed-epp.nominet.org.uk',
-	ote  => 'ote-epp.nominet.org.uk',
-);
+my $Host      = 'epp.nominet.org.uk';
+my $Hosttest  = 'testbed-epp.nominet.org.uk';
 my $EPPVer    = '1.0';
 my $EPPLang   = 'en';
 my $NSVer     = '2.0';
@@ -46,22 +50,22 @@ my $Debug     = 0;
 
 =head1 Name
 
-Net::EPP::Registry::Nominet - a simple client interface to the Nominet EPP
-service
+Net::EPP::Registry::Nominet - a simple client interface to Nominet EPP
+jobs
 
 =head1 Synopsis
 
+	#!/usr/bin/perl
 	use strict;
-	use warnings;
 	use Net::EPP::Registry::Nominet;
 
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
+	) or die ('Could not login to EPP server: ', $epp->get_error);
 
 	my $dom = 'foo.co.uk';
-
+	
 	if ($epp->check_domain($dom) == 1) {
 		print "Domain $dom is available\n" ;
 	} else {
@@ -80,10 +84,10 @@ service
 
 =head1 Description
 
-L<Nominet|https://www.nominet.uk/> is the organisation in charge of
+L<Nominet|http://www.nominet.org.uk/> is the organisation in charge of
 domain names under the .uk TLD.  Historically it used cryptographically
-signed email communications with registrars to provision domains.
-More recently (since 2010) it has instituted an EPP system
+signed email communications with registrars to provision domains (and
+still does). More recently (since 2010) it has instituted an EPP system
 which is sufficiently different from standard EPP that none of the
 standard modules will work seamlessly with it.
 
@@ -104,18 +108,17 @@ END {}
 	my $epp = Net::EPP::Registry::Nominet->new (
 		user  =>  'MYTAG',
 		pass  =>  'mypass'
-	) or die ('Could not login to EPP server: ', $Net::EPP::Registry::Nominet::Message);
+	) or die ('Could not login to EPP server: ', $epp->get_error);
 
 The constructor for C<Net::EPP::Registry::Nominet> has the same
-general form as the one for L<Net::EPP::Simple>, but with the following
+general form as the one for C<Net::EPP::Simple>, but with the following
 exceptions:
 
 =over
 
-=item * C<host> will be set to the appropriate endpoint. Specify C<ote>
-with a true value to connect to the OT&E endpoint, C<test> with a true
-value for the testbed endpoint and none of these for the standard live
-endpoint.
+=item * If C<test> is set but C<testssl> is not, C<port> defaults to 8700
+
+=item * if C<test> is set, C<host> defaults to 'testbed-epp.nominet.org.uk'. Otherwise C<host> defaults to 'epp.nominet.org.uk'.
 
 =item * C<timeout> defaults to 5 (seconds).
 
@@ -130,14 +133,11 @@ renewal command. It must be an integer between 1 and 10 inclusive (but
 note that renewing for 10 years pre-expiry will always fail because
 Nominet prohibits it).
 
-=item * C<login_opt> is a hashref of options to be passed directly
-through as the third optional argument to L<login()|/Login>.
-
 =item * There is no facility for a config file but this may be added in
 future versions.
 
-=item * There is no facility for supplying SSL client certificates
-because there is no support for them in the Nominet EPP server.
+=item * There are no SSL certificate options but these may be added in
+future versions.
 
 =back
 
@@ -151,42 +151,33 @@ sub new {
 	# to use with old version of Net::EPP.
 	$params{dom}		 = 1;
 
-	carp 'Nominet only supports port 700 these days - using that instead.'
-		if $params{port} && $params{port} != 700;
-	carp 'The testssl parameter is deprecated. ' .
-		'From August 2025 all connections use TLS.'
-		if exists $params{testssl};
-	$params{port} = 700; # Only option from August 2025
-	$params{ssl}  = 1;   # Only option from August 2025
-
 	if (defined $params{debug}) { $Debug = $params{debug}; }
-	$params{host} =
-		$params{test} ? $Host{test} :
-		$params{ote}  ? $Host{ote}  :
-		$Host{prod};
-
-	warn "Connecting to $params{host}:$params{port}\n" if $Debug;
-	$params{timeout}	= (int($params{timeout} || 0) > 0 ? $params{timeout} : 5);
-	if ($params{verify}) {
-		$params{SSL_ca_file}      ||= $params{ca_file};
-		$params{SSL_ca_path}      ||= $params{ca_path};
-		$params{SSL_verify_mode}  =   0x01;
+	if ($params{test} and $params{test} == 1) {
+		# Use test server
+		if ($params{testssl} and $params{testssl} == 1) {
+			$params{port} = 700;
+			$params{ssl}  =   1;
+		} else {
+			$params{port} = 8700;
+			$params{ssl}  = undef;
+		}
+		$params{host} = $Hosttest;
 	} else {
-		$params{SSL_verify_mode}  =   0x00;
-	}
-	if ($params{ssl} and $params{ciphers}) {
-		$params{SSL_cipher_list} = $params{ciphers};
+		# Use live server
+		$params{port} = 700;
+		$params{ssl}  =   1;
+		$params{host} = $Host;
 	}
+	$params{timeout}	= (int($params{timeout} || 0) > 0 ? $params{timeout} : 5);
 
 	my $self = Net::EPP::Client->new(%params);
 	unless ($self->{timeout}) { $self->{timeout} = $params{timeout}; }
-	$self->{debug} = $Debug > 1 ? 1 : 0; # for parent
 
 	# Set the default years.
 	$self->{def_years} = 2;
 	if (defined $params{def_years}) {
 		my $years = scalar $params{def_years};
-		if ($years =~ /^[0-9]+$/) {
+		if ($years and $years =~ /^[0-9]+$/) {
 			if ($years > 0 and $years < 11) {
 				$self->{def_years} = $years;
 			} else {
@@ -202,11 +193,17 @@ sub new {
 	bless($self, $class);
 
 	# Connect to server
-	$self->_go_connect (%params) or return;
+	eval { $self->{greeting} = $self->connect; };
+	unless ($self->{greeting}) {
+		$self->{connected} = 0;
+		warn 'No greeting returned: cannot continue';
+		return undef;
+	}
+	$self->{connected}      = 1;
 
 	# Login
 	unless (defined $params{login} and $params{login} == 0) {
-		$self->login ($params{user}, $params{pass}, $params{login_opt});
+		$self->login ($params{user}, $params{pass});
 	}
 
 	# If there was an error in the constructor, there's no point
@@ -214,25 +211,6 @@ sub new {
 	return $Error ? undef : $self;
 }
 
-sub _go_connect {
-	my ($self, %params) = @_;
-	if (scalar keys %params) {
-		$self->{connect_params} = \%params;
-	} else {
-		%params = %{$self->{connect_params}};
-	}
-
-	# Connect to server
-	eval { $self->{greeting} = $self->connect (%params); };
-	unless ($self->{greeting}) {
-		$self->{connected} = undef;
-		warn 'No greeting returned: cannot continue';
-		warn ($@) if $@;
-		return undef;
-	}
-	$self->{connected}      = 1;
-}
-
 =pod
 
 =head1 Login
@@ -243,39 +221,16 @@ The client can perform a standalone EPP Login if required.
 		or die ("Could not login: ", $epp->get_reason);
 
 The optional third argument, C<$opt_ref>, is a hash ref of login
-options. Currently the only supported option is 'tag_list' which
-should be set to a true value if the user needs to use the
-L<list_tags|/List Tags>
-method. Nominet operates a number of
-L<mutually exclusive schemas|https://registrars.nominet.uk/uk-namespace/registration-and-domain-management/schemas-and-namespaces/>
-so that the user needs to login again to perform different tasks. At present
-this module only supports two sets: the standard tasks and the tag list.
+options. Currently the only supported option is 'nominet_schemas' which
+should be set to a true value if the user requires the old, deprecated
+Nominet EPP schemas. These were removed from the Live systems before
+August 2013, so should not be retained in production code.
 
 =cut
 
 sub login {
 	my ($self, $user, $pass, $options) = @_;
 
-	if ($self->{authenticated}) {
-		$Error = 'Already logged in';
-		carp ($Error);
-		return;
-	}
-
-	unless (defined $user) {
-		$Error = 'No username (tagname) supplied';
-		carp ($Error);
-		return;
-	}
-
-	unless (defined $pass) {
-		$Error = 'No password supplied';
-		carp ($Error);
-		return;
-	}
-
-	$self->_go_connect () unless $self->connected;
-
 	# Set login frame
 	my $login = Net::EPP::Frame::Command::Login->new;
 
@@ -284,35 +239,36 @@ sub login {
 	$login->version->appendText($EPPVer);
 	$login->lang->appendText($EPPLang);
 
-	my @ns   = ();
-	my @svcs = ();
-	my $baseuri = 'http://www.nominet.org.uk/epp/xml';
+	my $objects = $self->{greeting}->getElementsByTagNameNS(EPP_XMLNS, 'objURI');
+	#while (my $object = $objects->shift) {
+	#for my $ns ('nom-account', 'nom-contact', 'nom-domain',
 
-	if ($options->{'tag_list'}) {
-		push @ns, 'nom-tag';
+
+	if ($options->{'nominet_schemas'}) {
+		# Deprecated
+		for my $ns ('nom-domain', 'nom-notifications') {
+			my $el = $login->createElement('objURI');
+			$el->appendText("http://www.nominet.org.uk/epp/xml/$ns-$NSVer");
+			$login->svcs->appendChild($el);
+		}
+		my $foo = $login->createElement('objURI');
+		$foo->appendText("urn:ietf:params:xml:ns:host-1.0");
+		$login->svcs->appendChild($foo);
 	} else {
-		push @ns, qw/epp eppcom domain host contact secDNS/;
-		push @svcs, qw/domain-nom-ext-1.2 contact-nom-ext-1.0
-			std-notifications-1.2 std-warning-1.1 std-contact-id-1.0
-			std-release-1.0 std-handshake-1.0 nom-abuse-feed-1.0
-			std-fork-1.0 std-list-1.0 std-locks-1.0 std-unrenew-1.0/;
-	}
-	# Standard schemas and extensions
-	for my $ns (@ns) {
-		my $el = $login->createElement('objURI');
-		my $ver = $EPPVer;
-		$ver = 1.1 if $ns eq 'secDNS';
-		my $text = "urn:ietf:params:xml:ns:$ns-$ver";
-		$text = "$baseuri/$ns-$ver" if $ns =~ /-/;
-		$el->appendText($text);
-		$login->svcs->appendChild($el);
-	}
-	# Extensions go here
-	if (scalar @svcs) {
+		# Standard schemas and extensions
+		for my $ns ('epp', 'eppcom', 'domain', 'host', 'contact') {
+			my $el = $login->createElement('objURI');
+			$el->appendText("urn:ietf:params:xml:ns:$ns-$EPPVer");
+			$login->svcs->appendChild($el);
+		}
+		# Extensions go here
 		my $ext = $login->createElement('svcExtension');
-		for my $ns (@svcs) {
+		for my $ns (qw/domain-nom-ext-1.2 contact-nom-ext-1.0
+		std-notifications-1.2 std-warning-1.1 std-contact-id-1.0
+		std-release-1.0 std-handshake-1.0 nom-abuse-feed-1.0
+		std-fork-1.0 std-list-1.0 std-locks-1.0 std-unrenew-1.0/) {
 			my $el = $login->createElement('extURI');
-			$el->appendText("$baseuri/$ns");
+			$el->appendText("http://www.nominet.org.uk/epp/xml/$ns");
 			$ext->appendChild($el);
 		}
 		$login->svcs->appendChild($ext);
@@ -326,37 +282,19 @@ sub login {
 	}
 
 	$self->{authenticated} = 1;
-	$self->{login_params} = [$user, $pass, $options];
 	return $self;
 }
 
-sub logout {
-	my $self = shift;
-	my $res = $self->SUPER::logout (@_);
-	$self->{authenticated} = 0 if $res;
-	return $res;
-}
-
 =head1 Availability checks
 
-The availability checks work similarly to L<Net::EPP::Simple> except
-that in list context they return an array with up to three elements.
-The first element is the
+The availability checks work similarly to C<Net::EPP::Simple> except
+that they return an array with two elements. The first element is the
 availability indicator as before (0 if provisioned, 1 if available,
-undef on error). The second element is the abuse counter which shows
-how many more such checks you may run.
-The third element gives the reason for the lack of availability, if any.
+undef on error) and the second element is the abuse counter which shows
+how many more such checks you may run. This counter is only relevant
+for check_domain and will always be undef for the other check methods.
 
-These two extra fields are only relevant for check_domain and will always
-be undef for the other check methods.
-
-	# List context
-	my ($avail, $left, $reason) = $epp->check_domain ("foo.uk");
-	($avail) = $epp->check_contact ("ABC123");
-	($avail) = $epp->check_host ("ns0.foo.co.uk");
-
-	# Scalar context
-	$avail = $epp->check_domain ("foo.uk");
+	my ($avail, $left) = $epp->check_domain ("foo.co.uk");
 	$avail = $epp->check_contact ("ABC123");
 	$avail = $epp->check_host ("ns0.foo.co.uk");
 
@@ -389,19 +327,13 @@ sub _check {
 	}
 
 	my $response = $self->_send_frame ($frame) or return undef;
-	my $avail = $response->getNode($spec[1], $key)->getAttribute('avail');
-	return $avail unless wantarray;
 
 	my $extra = $response->getNode("$type-nom-ext:chkData");
 	my $count = undef;
 	$count = $extra->getAttribute('abuse-limit') if defined $extra;
 	warn "Remaining checks = $count\n" if ($Debug and defined $count);
 
-	$extra = $response->getNode($spec[1], 'reason');
-	my $reason;
-	$reason = $extra->textContent if defined $extra;
-
-	return ($avail, $count, $reason);
+	return ($response->getNode($spec[1], $key)->getAttribute('avail'), $count);
 }
 
 =head1 Domain Renewal
@@ -431,18 +363,6 @@ be post-expiry.
 
 sub renew {
 	my ($self, $renew) = @_;
-	unless (defined $renew) {
-		carp "No argument provided";
-		return undef;
-	}
-	unless (ref $renew and ref $renew eq 'HASH') {
-		carp "Argument to renew is not a hash reference";
-		return undef;
-	}
-	unless ($renew->{name}) {
-		carp "Argument to renew has no 'name' field";
-		return undef;
-	}
 	my $domain = $renew->{name};
 	my $expiry = $renew->{cur_exp_date};
 	my $years  = $renew->{period};
@@ -516,7 +436,7 @@ sub unrenew {
 	$frame->getCommandNode->appendChild ($elem);
 
 	if (my $response = $self->_send_frame ($frame)) {
-
+	
 		# Results not necessarily returned by EPP in the same order.
 		# Construct a hash ref with domains as keys and expiry dates as
 		# values
@@ -594,15 +514,7 @@ registration.
 		nameservers  => {
 			'nsname0'  => "ns1.bar.co.uk",
 			'nsname1'  => "ns2.bar.co.uk"
-		},
-		secDNS       => [
-			{
-				keyTag     => 123,
-				alg        => 5,
-				digestType => 1,
-				digest     => '8A9CEBB665B78E0142F1CEF47CC9F4205F600685'
-			}
-		]
+		}
 	};
 	my ($res) = $epp->create_domain ($domain);
 	if ($res) {
@@ -691,21 +603,6 @@ sub create_domain {
 	$name->appendChild ($pw);
 	$obj->appendChild ($name);
 
-
-	# DNSSEC
-	my $exttype = 'secDNS';
-	if ($domain->{$exttype}) {
-		my @spec  = $self->spec ($exttype);
-		my $obj2  = $frame->addObject (@spec);
-		for my $dsrec (@{$domain->{$exttype}}) {
-			$self->_add_dsrec ($obj2, $frame, $dsrec);
-		}
-
-		my $extension = $frame->command->new ('extension');
-		$extension->appendChild ($obj2);
-		$frame->command->insertAfter ($extension, $frame->getCommandNode);
-	}
-
 	# Request complete, so send the frame
 	$frame->getCommandNode->appendChild ($obj);
 
@@ -729,10 +626,7 @@ account like this to perform the registration.
 		'trad-name'   => 'Examples4u',
 		'type'        => 'LTD',
 		'co-no'	      => '12345678',
-		'disclose'    => {
-			'org'  => 1,
-			'addr' => 0
-		},
+		'opt-out'     => 'n',
 		'postalInfo'  => { loc => {
 			'name'  => 'Arnold N Other',
 			'org'   => 'Example Company',
@@ -777,7 +671,7 @@ sub create_contact {
 	# Extensions
 	my @spec = $self->spec ('contact-nom-ext');
 	my $obj  = $frame->addObject (@spec);
-	for my $field (qw/ trad-name type co-no /) {
+	for my $field (qw/ trad-name type co-no opt-out /) {
 		next unless ($contact->{$field});
 		my $name = $frame->createElement("contact-nom-ext:$field");
 		$name->appendText ($contact->{$field});
@@ -787,19 +681,6 @@ sub create_contact {
 	$extension->appendChild ($obj);
 	$frame->command->insertAfter ($extension, $frame->getCommandNode);
 
-	if (defined $contact->{disclose}) {
-		my $add = $frame->createElement ('contact:disclose');
-		$add->setAttribute('flag', '1');
-		for my $field (qw/org addr/) {
-			next unless $contact->{disclose}->{$field};
-			my $disc = $frame->createElement ("contact:$field");
-			$disc->setAttribute('type', 'loc');
-			$add->appendChild ($disc);
-		}
-		$frame->getNode('create')->getChildNodes->shift->appendChild ($add)
-			if $add->hasChildNodes;
-	}
-
 	my $response = $self->_send_frame ($frame);
 	return $Code == 1000 ? 1 : undef;
 }
@@ -844,18 +725,6 @@ sub _add_nsaddr {
 	return;
 }
 
-sub _add_dsrec {
-	my ($self, $name, $frame, $dsrec) = @_;
-	my $ds = $frame->createElement ('secDNS:dsData');
-	for my $key (qw/ keyTag alg digestType digest /) {
-		my $field = $frame->createElement ("secDNS:$key");
-		$field->appendText ($dsrec->{$key});
-		$ds->appendChild ($field);
-	}
-	$name->appendChild ($ds);
-	return;
-}
-
 =head1 Modify objects
 
 The domains, contacts and hosts once created can be modified using
@@ -871,6 +740,8 @@ changes like this:
 		'add'          => { ns => ['ns1.newhost.com', 'ns2.newhost.com'] },
 		'rem'          => { ns => ['ns1.oldhost.net', 'ns2.oldhost.net'] },
 		'chg'          => {},
+		'first-bill'   => 'th',
+		'recur-bill'   => 'th',
 		'auto-bill'    => 21,
 		'auto-period'  => 5,
 		'next-bill'    => '',
@@ -883,26 +754,11 @@ You cannot use C<chg> to change nameservers or extension fields. The C<chg>
 entry is only used to move a domain between registrants with the same
 name.
 
-The C<add> and C<rem> groups are also used to add and remove DS records.
-eg:
-
-	my $changes = {
-		'name'         => 'foo.co.uk',
-		'add'          => {
-			secDNS => [{
-				keyTag     => 25103,
-				alg        => 5,
-				digestType => 1,
-				digest     => '8A9CEBB665B78E0142F1CEF47CC9F4205F600685'
-			}]
-		},
-		'rem'          => {}
-	};
-
-The extension fields can only be set outside of the add, rem and
-chg fields. The supported extensions in this module are: C<auto-bill>,
-C<auto-period>, C<next-bill>, C<next-period>, C<renew-not-required> and
-C<notes>. All of these are scalars aside from C<notes> which is an array ref.
+The extension fields can only be set outside of the add, rem and chg
+fields. The supported extensions in this module are:
+first-bill, recur-bill, auto-bill, auto-period, next-bill, next-period
+and notes. All of these are scalars aside from notes which is an array
+ref.
 
 C<update_domain()> returns undef on failure, 1 on success.
 
@@ -935,20 +791,12 @@ sub modify_domain {
 	for my $action ('add', 'rem', 'chg') {
 		if ($data->{$action}) {
 			$name = $frame->createElement ("domain:$action");
-			if ($action ne 'chg' && $data->{$action}->{ns}) {
+			if ($data->{$action}->{ns}) {
 				my $name2 = $frame->createElement ("domain:ns");
 				for my $ns (@{$data->{$action}->{ns}}) {
 					$self->_add_nsname ($name2, $frame, $ns);
 				}
 				$name->appendChild ($name2);
-			} elsif ($action eq 'chg') {
-				if ($data->{$action}->{registrant}) {
-					my $name2 = $frame->createElement ("domain:registrant");
-					$name2->appendText ($data->{$action}->{registrant});
-					$name->appendChild ($name2);
-				} else {
-					carp "'chg' is present but no 'registrant' field";
-				}
 			}
 			$obj->appendChild ($name);
 		}
@@ -958,8 +806,8 @@ sub modify_domain {
 	# Extensions
 	@spec = $self->spec ('domain-nom-ext');
 	$obj  = $frame->addObject (@spec);
-	for my $field (qw/ auto-bill auto-period
-			next-bill next-period renew-not-required reseller /) {
+	for my $field (qw/ first-bill recur-bill auto-bill auto-period
+			next-bill next-period reseller /) {
 		next unless ($data->{$field});
 		my $name = $frame->createElement("domain-nom-ext:$field");
 		$name->appendText ($data->{$field});
@@ -972,26 +820,8 @@ sub modify_domain {
 			$obj->appendChild ($name);
 		}
 	}
-	# DNSSEC
-	# Nominet does not support MaxSigLife which is the only possible use
-	# for 'chg', so do not cater for it here (yet).
-	#
-	my $exttype = 'secDNS';
-	@spec  = $self->spec ($exttype);
-	my $obj2  = $frame->addObject (@spec);
-	for my $action ('rem', 'add') {
-		if ($data->{$action}->{$exttype}) {
-			$name = $frame->createElement ("$exttype:$action");
-			for my $dsrec (@{$data->{$action}->{$exttype}}) {
-				$self->_add_dsrec ($name, $frame, $dsrec);
-			}
-			$obj2->appendChild ($name);
-		}
-	}
-
 	my $extension = $frame->command->new ('extension');
 	$extension->appendChild ($obj);
-	$extension->appendChild ($obj2);
 	$frame->command->insertAfter ($extension, $frame->getCommandNode);
 
 	my $response = $self->_send_frame ($frame);
@@ -1002,7 +832,7 @@ sub modify_domain {
 =head2 Modify contacts
 
 To modify a contact, which includes aspects of the registrant such as
-the disclose flags etc., you will again need to create a hashref of the
+the WHOIS opt-out etc., you will again need to create a hashref of the
 changes like this:
 
 	my $changes = {
@@ -1010,6 +840,7 @@ changes like this:
 		'type'        =>  'FCORP',
 		'trad-name'   =>  'American Industries',
 		'co-no'       =>  '99998888',
+		'opt-out'     =>  'N',
 		'postalInfo'  => {
 			'loc' => {
 				'name' => 'James Johnston',
@@ -1023,14 +854,11 @@ changes like this:
 			}
 		},
 		'voice'	=>	'+1.77777776666',
-		'email'	=>	'jj@example.com',
-		'disclose' => {
-			'addr' => 1
-		}
+		'email'	=>	'jj@example.com'
 	};
 	my $res = $epp->update_contact ($changes) or die $epp->get_reason;
 
-Note that this differs from the syntax of L<Net::EPP::Simple> where that
+Note that this differs from the syntax of C<Net::EPP::Simple> where that
 takes the stock C<add>, C<rem> and C<chg> elements.
 
 It returns undef on failure, 1 on success.
@@ -1039,18 +867,6 @@ There is also a convenience method C<modify_contact()> which takes the
 contact id as the first argument and the hashref of changes as the
 second argument.
 
-Note that due to an undocumented restriction in Nominet's EPP servers
-it is not possible to modify the disclose flags for both addr and org
-to different values in one request.
-
-If the hashref contains the key C<new-id> like so:
-
-    my $changes = { id => 'ABC123', 'new-id' => 'XYZ789' };
-
-then the ID of the contact will be changed to the new ID (which must
-be unique in the entire registry). In this case any other fields in the
-hashref will be ignored.
-
 =cut
 
 sub update_contact {
@@ -1061,11 +877,7 @@ sub update_contact {
 sub modify_contact {
 	my ($self, $cont, $data) = @_;
 
-	# If given a 'new-id', update that and nothing else
-	return $self->_modify_contact_id ($cont, $data->{'new-id'})
-		if exists $data->{'new-id'};
-
-	# Sort out the contact to be updated
+	# Sort out the domain to be updated
 	my $frame;
 	my @spec = $self->spec ('contact');
 	$frame = Net::EPP::Frame::Command::Update->new;
@@ -1080,7 +892,7 @@ sub modify_contact {
 	#   my $frame = $self->_generate_update_contact_frame($data);
 	# but it won't work because of present, but empty, add/rem/chg
 	# elements. Equally we cannot do this:
-	#   my $frame = Net::EPP::Frame::Command::Update::Contact->new;
+    #   my $frame = Net::EPP::Frame::Command::Update::Contact->new;
 	#   $frame->setContact ( $cont );
 	# so instead it needs this extra chunk of code which follows:
 
@@ -1131,37 +943,12 @@ sub modify_contact {
 		$elem->appendText ($data->{$field});
 		$chg->appendChild ($elem);
 	}
-	if (defined $data->{disclose}) {
-		# Return an error if there's a mix of flags
-		my @flags = values (%{$data->{disclose}});
-		if ($#flags > 0 && $flags[0] != $flags[1]) {
-			$Error = "Nominet prohibits adding and removing disclosures " .
-				"in one action";
-			return;
-		}
-		# This doesn't need to be so complicated but it's staying this
-		# way in case Nominet decide to allow a mix of actions in the
-		# future.
-		my $add = $frame->createElement ('contact:disclose');
-		$add->setAttribute('flag', '1');
-		my $del = $frame->createElement ('contact:disclose');
-		$del->setAttribute('flag', '0');
-		for my $field (qw/org addr/) {
-			next unless defined $data->{disclose}->{$field};
-			my $disc = $frame->createElement ("contact:$field");
-			$disc->setAttribute('type', 'loc');
-			($data->{disclose}->{$field} ? $add : $del)->appendChild ($disc);
-		}
-		for my $child ($add, $del) {
-			$chg->appendChild ($child) if $child->hasChildNodes;
-		}
-	}
 	if ($chg->hasChildNodes) { $obj->appendChild($chg); }
 
 	# Extensions
 	@spec = $self->spec ('contact-nom-ext');
 	$obj  = $frame->addObject (@spec);
-	for my $field (qw/ trad-name type co-no /) {
+	for my $field (qw/ trad-name type co-no opt-out /) {
 		next unless ($data->{$field});
 		my $name = $frame->createElement("contact-nom-ext:$field");
 		$name->appendText ($data->{$field});
@@ -1175,26 +962,6 @@ sub modify_contact {
 	return $Code == 1000 ? 1 : undef;
 }
 
-sub _modify_contact_id {
-	my ($self, $old_id, $new_id) = @_;
-
-	my @spec  = $self->spec ('contact-id');
-	my $frame = Net::EPP::Frame::Command::Update->new;
-	my $obj   = $frame->addObject (@spec);
-	my $id    = $frame->createElement ('contact-id:id');
-	$id->appendText ($old_id);
-	$obj->appendChild ($id);
-	my $chg = $frame->createElement ('contact-id:chg');
-	$id     = $frame->createElement ('contact-id:id');
-	$id->appendText ($new_id);
-	$chg->appendChild ($id);
-	$obj->appendChild ($chg);
-
-	$frame->getCommandNode->appendChild ($obj);
-	my $response = $self->_send_frame ($frame);
-	return $Code == 1000 ? 1 : undef;
-}
-
 =head2 Modify nameservers
 
 To modify a nameserver, you will need to create a hashref of the
@@ -1255,59 +1022,6 @@ sub modify_host {
 	return $Code == 1000 ? 1 : undef;
 }
 
-=head1 Fork contact
-
-	my $res = $epp->fork ($old_id, $new_id, @domains);
-
-Splitting out some domains on a contact to a copy of that contact can be
-achieved using C<fork()>. The first optional argument is the existing
-contact ID. If this is undef then the existing contact will be that on
-the listed domains.
-
-The second optional argument is the ID of the new contact to create. If
-this is undef then a random ID will be assigned by Nominet.
-
-The third and subsequent arguments are the domain names to be moved from the
-existing contact to the new.
-
-Returns the new contact ID on success, undef otherwise.
-
-=cut
-
-sub fork {
-	my ($self, $old_id, $new_id, @doms) = @_;
-
-	my $type   = 'f';
-	my @spec   = $self->spec ($type);
-	my $frame  = Net::EPP::Frame::Command::Update->new;
-
-	my $obj    = $frame->createElement ('f:fork');
-	$obj->setAttribute ("xmlns:$type", $spec[1]);
-
-	if (defined $old_id) {
-		my $id = $frame->createElement ('f:contactId');
-		$id->appendText ($old_id);
-		$obj->appendChild ($id);
-	}
-	if (defined $new_id) {
-		my $id = $frame->createElement ('f:newContactId');
-		$id->appendText ($new_id);
-		$obj->appendChild ($id);
-	}
-	for my $dom (@doms) {
-		my $elem = $frame->createElement ('f:domainName');
-		$elem->appendText ($dom);
-		$obj->appendChild ($elem);
-	}
-
-	$frame->getCommandNode->appendChild ($obj);
-	my $response = $self->_send_frame ($frame);
-	return undef unless $Code == 1000;
-	my $id = $response->getNode ('contact:id')->firstChild->toString ();
-	warn "Forked contact ID is $id\n" if $Debug;
-	return $id;
-}
-
 =head1 Querying objects
 
 The interface for querying domains, contacts and hosts is the same as
@@ -1358,42 +1072,37 @@ sub _info {
 
 	if ($type eq 'domain') {
 		my $extra = $response->getNode('domain-nom-ext:infData');
-		my $secdns = $response->getNode('secDNS:infData');
-		return $self->_domain_infData_to_hash($infData, $extra, $secdns);
+		return $self->_domain_infData_to_hash($infData, $extra);
 	} elsif ($type eq 'contact') {
-		# Grab disclose infdata before Net::EPP::Simple deletes it
-		my $disclose = $self->_disclose_infData_to_hash ($infData);
-		$self->_clean_addr($infData);
 		my $this = $self->_contact_infData_to_hash($infData);
+		# Strip out the strange empty addr entries created by
+		# Net::EPP::Simple
+		for my $subtype ('int', 'loc') {
+			delete $this->{postalInfo}->{$subtype}->{addr}->{''} if
+				$this->{postalInfo}->{$subtype};
+		}
 		# Add in the Nominet extras (reg, rather than contact)
 		my $extra = $response->getNode('contact-nom-ext:infData');
-		return $self->_merge_contact_infData ($this, $extra, $disclose);
+		return $self->_merge_contact_infData ($this, $extra);
 	} elsif ($type eq 'host') {
 		return $self->_host_infData_to_hash($infData);
 	}
 }
 
 sub _domain_infData_to_hash {
-	my ($self, $infData, $extra, $secdns) = @_;
+	my ($self, $infData, $extra) = @_;
 
 	my $hash = $self->_node_to_hash ($infData, ['registrant',
 		'clID', 'crID', 'crDate', 'exDate', 'name', 'roid']);
 
-	my $extrahash = $self->_node_to_hash ($extra, [
-	'auto-bill', 'next-bill', 'auto-period',
-	'next-period', 'reg-status', 'renew-not-required', 'notes', 'reseller']);
+	my $extrahash = $self->_node_to_hash ($extra, ['first-bill',
+	'recur-bill', 'auto-bill', 'next-bill', 'auto-period',
+	'next-period', 'reg-status', 'notes', 'reseller']);
 
 	for (keys %$extrahash) {
 		$hash->{$_} = $extrahash->{$_};
 	}
 
-	if ($secdns) {
-		my $dsObjs = $secdns->nonBlankChildNodes;
-		while (my $dsObj = $dsObjs->shift) {
-			push @{$hash->{secDNS}}, $self->_node_to_hash ($dsObj);
-		}
-	}
-
 	my $hostObjs = $infData->getElementsByLocalName('hostObj');
 	while (my $hostObj = $hostObjs->shift) {
 		push(@{$hash->{ns}}, $hostObj->textContent);
@@ -1402,46 +1111,15 @@ sub _domain_infData_to_hash {
 	return $hash;
 }
 
-sub _disclose_infData_to_hash {
-	my ($self, $infData) = @_;
-	my $disc = $infData->getElementsByLocalName('disclose')->shift;
-	return unless $disc;
-	my $flag = $disc->getAttribute('flag');
-	my $hash;
-	for my $child ($disc->getChildrenByTagName('*')) {
-		$hash->{$child->localname} = $flag;
-	}
-	return $hash;
-}
-
-sub _tag_infData_to_hash {
-	my ($self, $infData) = @_;
-	return $self->_node_to_hash ($infData, ['registrar-tag',
-		'name', 'handshake', 'trad-name']);
-}
-
-sub _clean_addr {
-	my ($self, $infData) = @_;
-	# Remove the whitespace text nodes from the addresses as
-	# Net::EPP::Simple does not handle them nicely
-	for my $addr ($infData->getElementsByTagName('contact:addr')) {
-		for my $child ($addr->childNodes) {
-			$addr->removeChild ($child)
-				unless $child->nodeType == XML::LibXML::XML_ELEMENT_NODE;
-		}
-	}
-}
-
 sub _merge_contact_infData {
-	my ($self, $old, $extra, $disclose) = @_;
+	my ($self, $old, $extra) = @_;
 
 	my $extrahash = $self->_node_to_hash ($extra, ['type',
-	'co-no', 'trad-name']);
+	'co-no', 'opt-out', 'trad-name']);
 
 	for (keys %$extrahash) {
 		$old->{$_} = $extrahash->{$_};
 	}
-	$old->{disclose} = $disclose;
 	return $old;
 
 }
@@ -1511,56 +1189,6 @@ sub list_domains {
 	return $domlist;
 }
 
-=head2 List Tags
-
-When transferring domains it may be useful to have a list of possible
-tag names. This method returns the full list of tags as an array ref.
-Each entry in the arrayref is itself a hashref with these keys:
-
-=over
-
-=item C<registrar-tag> is the tag name to use in release actions, etc.
-
-=item C<name> is the name of the registrar for display purposes
-
-=item C<trad-name> is the trading name of the registrar (may be empty
-string)
-
-=item C<handshake> is "Y" if they require handshakes on transfer
-or "N" otherwise
-
-=back
-
-	my $taglist = $epp->list_tags;
-
-It accepts no arguments and returns undef on error.
-
-Note that you must have passed the C<tag_list> option to L<login()|/Login>
-in order to use this method.
-
-=cut
-
-sub list_tags {
-	my $self = shift;
-	my $type = 'tag';
-	my @spec = $self->spec ($type);
-	my $frame = Net::EPP::Frame::Command::Info->new;
-	my $name = $frame->createElement ('tag:list');
-	$name->setAttribute ("xmlns:$type", $spec[1]);
-	$frame->getCommandNode->appendChild ($name);
-
-	my $response = $self->_send_frame($frame) or return undef;
-	if ($Code != 1000) { return undef; }
-
-	my $infData = $response->getNode($spec[1], 'listData');
-	my $taglist = [];
-	for my $node ($infData->childNodes) {
-		next unless $node->nodeType == XML::LibXML::XML_ELEMENT_NODE;
-		push @$taglist, $self->_tag_infData_to_hash ($node);
-	}
-	return $taglist;
-}
-
 =head1 Hello
 
 EPP allows the use of a "hello" operation which effectively tests that
@@ -1569,28 +1197,28 @@ reset any inactivity timeout which the server might apply. Nominet's
 documentation seems to indicate a 60 minute timeout (as at August 2013).
 
 	my $res = $epp->hello ();
-
+	
 The hello method takes no arguments. It returns 1 on success, undef
 otherwise.
 
 This performs much the same function as the ping method of
-L<Net::EPP::Simple> (which could be used instead) but provides more
+Net:EPP::Simple (which could be used instead) but provides more
 extensive error handling.
 
 =cut
 
 sub hello {
 	my $self = shift;
-	unless ($self->connected) {
+	unless ($self->{connected}) {
 		warn "Hello attempt while disconnected\n" if $Debug;
 		return undef;
 	}
-	my $frame = Net::EPP::Frame::Hello->new;
+	my $frame = Net::EPP::Frame::Hello->new->toString;
 
-	warn "Sending XML = \n" . $frame . "\n" if $Debug > 1;
+	warn "Sending XML = \n" . $frame . "\n" if $Debug;
 	my $greeting = $self->request($frame);
-	warn "Response XML = \n" . $greeting->toString() . "\n"
-		if ($Debug > 1 && defined $greeting);
+	warn "Response XML = \n" . $greeting->toString() . "\n" if ($Debug
+	and defined $greeting);
 
 	unless ($greeting) {
 		$Error = sprintf("Server returned a %d code", $Code);
@@ -1612,8 +1240,8 @@ array of type, XMLNS and XSI for use with various frame and XML
 routines. It is not expected to be called independently by the user but
 is here if you need it.
 
-Type can currently be one of: domain, contact, contact-ext, contact-id
-host, l (for list), u (for unrenew), r (for release), f (for fork)
+Type can currently be one of: domain, contact, contact-ext,
+host, l (for list), u (for unrenew), r (for release)
 
 	my @spec = $epp->spec ('domain');
 
@@ -1634,12 +1262,6 @@ sub spec {
 			'http://www.nominet.org.uk/epp/xml/domain-nom-ext-1.2',
 			'http://www.nominet.org.uk/epp/xml/domain-nom-ext-1.2 domain-nom-ext-1.2.xsd');
 	}
-	if ($type eq 'secDNS') {
-		my $ver = 1.1;
-		return ($type,
-			"urn:ietf:params:xml:ns:secDNS-$ver",
-			"urn:ietf:params:xml:ns:secDNS-$ver secDNS-$ver.xsd");
-	}
 	if ($type eq 'contact') {
 		return ($type,
 			"urn:ietf:params:xml:ns:contact-$EPPVer",
@@ -1650,11 +1272,6 @@ sub spec {
 			'http://www.nominet.org.uk/epp/xml/contact-nom-ext-1.0',
 			'http://www.nominet.org.uk/epp/xml/contact-nom-ext-1.0 contact-nom-ext-1.0.xsd');
 	}
-	if ($type eq 'contact-ext' or $type eq 'contact-id') {
-		return ($type,
-			'http://www.nominet.org.uk/epp/xml/std-contact-id-1.0',
-			'http://www.nominet.org.uk/epp/xml/std-contact-id-1.0 std-contact-id-1.0.xsd');
-	}
 	if ($type eq 'host') {
 		return ($type,
 			"urn:ietf:params:xml:ns:host-$EPPVer",
@@ -1675,16 +1292,6 @@ sub spec {
 			"http://www.nominet.org.uk/epp/xml/std-release-1.0",
 			"http://www.nominet.org.uk/epp/xml/std-release-1.0 std-release-1.0.xsd");
 	}
-	if ($type eq 'f') {
-		return ($type,
-			"http://www.nominet.org.uk/epp/xml/std-fork-1.0",
-			"http://www.nominet.org.uk/epp/xml/std-fork-1.0 std-fork-1.0.xsd");
-	}
-	if ($type eq 'tag') {
-		return ($type,
-			"http://www.nominet.org.uk/epp/xml/nom-tag-$EPPVer",
-			"http://www.nominet.org.uk/epp/xml/nom-tag-$EPPVer nom-tag-$EPPVer.xsd");
-	}
 }
 
 =head2 valid_voice
@@ -1785,19 +1392,6 @@ sub set_reason {
 	return $self->{'_reason'};
 }
 
-sub get_debug {
-	my $self = shift;
-	return $self->{debug};
-}
-
-sub set_debug {
-	my ($self, $debug) = @_;
-	croak "Debug must be whole number" unless $debug =~ /^\d+$/;
-	$Debug = $debug;
-	$self->{debug} = $Debug > 1 ? 1 : 0; # for parent
-	return $Debug;
-}
-
 sub _send_frame {
 	my ($self, $frame) = @_;
 
@@ -1808,21 +1402,6 @@ sub _send_frame {
 		$Code   = 0;
 		$Error  = "No response from server";
 		warn $Error;
-		$self->{connected}     = undef;
-		$self->{authenticated} = 0;
-		if ($self->{reconnect}) {
-			# Attempt to reconnect
-			for (1 .. $self->{reconnect}) {
-				$self->_go_connect and
-				$self->login (@{$self->{login_params}});
-				if ($self->{authenticated}) {
-					$frame->clTRID->firstChild->setData ('');
-					return $self->_send_frame ($frame);
-				}
-				warn "Re-connection attempt $_ of $self->{reconnect} failed.\n";
-				sleep 2;
-			}
-		}
 		return undef;
 	}
 	warn "Response = " . $response->toString . "\n" if $Debug > 1;
@@ -1847,12 +1426,14 @@ sub _send_frame {
 
 =head1 TODO
 
-=encoding utf8
-
 =over
 
-=item * The poll, handshake, lock and reseller operations
-are not yet supported.
+=item * The poll, fork, handshake, lock, tag list and reseller operations
+are not yet supported. Nor is there any DNSSEC facility yet.
+
+=item * Retrieving contact info can generate warnings from the
+superclass. Ideally this processing should be done locally (within this
+class) to avoid these warnings.
 
 =item * Much more extensive tests should be performed.
 
@@ -1865,12 +1446,12 @@ are not yet supported.
 =item * L<Net::EPP::Simple>
 
 =item * Nominet's L<EPP
-Documentation|https://registrars.nominet.uk/uk-namespace/registration-and-domain-management/registration-systems/epp/>
+Documentation|http://registrars.nominet.org.uk/registration-and-domain-management/registrar-systems/epp>
 
-=item * The EPP RFCs: L<RFC 5730|https://tools.ietf.org/html/rfc5730>,
-L<RFC 5731|https://tools.ietf.org/html/rfc5731>,
-L<RFC 5732|https://tools.ietf.org/html/rfc5732> and
-L<RFC 5733|https://tools.ietf.org/html/rfc5733>.
+=item * The EPP RFCs: L<RFC 5730|http://tools.ietf.org/html/rfc5730>, 
+L<RFC 5731|http://tools.ietf.org/html/rfc5731>,
+L<RFC 5732|http://tools.ietf.org/html/rfc5732> and
+L<RFC 5733|http://tools.ietf.org/html/rfc5733>.
 
 =back
 
@@ -1880,7 +1461,7 @@ Pete Houston <cpan@openstrike.co.uk>
 
 =head1 Licence
 
-This software is copyright © 2013-2025 by Pete Houston. It is released
+This software is copyright (c) 2013 by Pete Houston. It is released
 under the Artistic Licence (version 2) and the
 GNU General Public Licence (version 2).
 
